;***************************************************
; Program : 93C46 include file 
; Description : All subroutine of 93C46
; Filename : EE9346_1.asm
;***************************************************
; WRITE SUBROUTINE ;
WRITE	BSF	PORTA,3			; CS on
	CALL	EWEN				; Write enabled
	BCF	PORTA,3			; CS off
	NOP					; Delay 1us

	BSF	PORTA,3			; CS on
	MOVF	ADDRESS,W		; Send address first
	MOVWF	DATA_L	
	BCF	DATA_L,7			; Set command
	BSF	DATA_L,6			; Write command
	BSF	PORTA,1			; Send start bit
	BSF	PORTA,2			; Clock
	BCF	PORTA,2		
	CALL	SENDBYTE			; Send command & ADDRESS
	MOVF	H_BYTE,W			; Send data high byte
	MOVWF	DATA_L
	CALL	SENDBYTE
	MOVF	L_BYTE,W			; Send data low byte
	MOVWF	DATA_L
	CALL	SENDBYTE
	BCF	PORTA,3			; CS off
	NOP					; Delay 1us

	BSF	PORTA,3			; CS on
CHK_WR	BTFSS	PORTA,0			; Write complete?
	GOTO	CHK_WR			; No
	BCF	PORTA,3			; Yes CS off
	NOP					; Delay 1us
	BSF	PORTA,3			; CS on
	CALL	EWDS				; Write disabled
	BCF	PORTA,3			; CS off
	NOP					; Delay 1us
	RETURN

; READ SUBROUTINE ;
READ	BSF	PORTA,3			; CS on
	MOVF	ADDRESS,W		; Send address
	MOVWF	DATA_L		
	BSF	DATA_L,7			; Set command
	BCF	DATA_L,6			; Read command
	BSF	PORTA,1			; start bit
	BSF	PORTA,2			; Clock
	BCF	PORTA,2
	CALL	SENDBYTE			; Send command & address
	CALL	GET_DATA			; Read data high byte
	MOVF	DATA_L,W			; Save data high byte
	MOVWF	DATA_H		
	CALL	GET_DATA			; Read data low byte
	BCF	PORTA,3			; CS off
	NOP					; Delay 1us
	RETURN

; EEPROM WRITE enabled ;
EWEN	MOVLW	0x30				; Set command	
	MOVWF	DATA_L				; Write enabled command
	BSF	PORTA,1			; Send start bit
	BSF	PORTA,2			; Clock
	BCF	PORTA,2
	CALL	SENDBYTE			; Send command
	RETURN

; EEPROM WRITE DISABLED ;
EWDS	MOVLW	0x00				; Set command
	MOVWF	DATA_L				; Write disabled command
	BSF	PORTA,1			; Send start bit
	BSF	PORTA,2			; Clock
	BCF	PORTA,2
	CALL	SENDBYTE			; Send command
	RETURN

; ERASE ALL DATA ;
ERAL	BSF	PORTA,3			; CS on
	CALL	EWEN				; Write enabled
	BCF	PORTA,3			; CS off
	NOP					; Delay 1us

	BSF	PORTA,3				; CS on
	MOVLW	0x20
	MOVWF	DATA_L
	BSF	PORTA,1
	BSF	PORTA,2
	BCF	PORTA,2
	CALL	SENDBYTE
	BCF	PORTA,3				; CS off
	NOP						; Delay 1us

	BSF	PORTA,3				; CS on
CHK_ERA	BTFSS	PORTA,0				; Erase complete? 
	GOTO	CHK_ERA				; No ! loop
	BCF	PORTA,3				; CS off
	NOP						; Delay 1us
	
	BSF	PORTA,3				; CS on
	CALL	EWDS					; Write disabled
	BCF	PORTA,3				; CS off
	NOP						; Delay 1us

	RETURN

; SEND DATA TO EEPROM ;
SENDBYTE			
	MOVLW	0x08					; Counter = 8
	MOVWF	COUNT
SBIT	CALL	SENDBIT				; Send bit
	DECFSZ	COUNT,F				; Test counter = 0 ?
	GOTO	RL_DATA				; No
	RETURN
RL_DATA	RLF	DATA_L,F				; Shift bit
	GOTO	SBIT

SENDBIT	BCF	PORTA,1				; Clear DI
	BTFSC	DATA_L,7				; If bit = 1 then set RA1
	BSF	PORTA,1				 
	BSF	PORTA,2				; Clock
	BCF	PORTA,2
	RETURN

; READ DATA FROM EEPROM ;
GET_DATA			
	MOVLW	0x08					; Counter = 8
	MOVWF	COUNT
SHIFT	BSF	PORTA,2				; Clock
	BCF	PORTA,2
	MOVF	PORTA,W				; Read data from EEPROM
	MOVWF	TEMP					; Save data
	RRF	TEMP,F			; Rotate data to Carry flag	
	RLF	DATA_L,F			; Rotate Carry flag to data
	DECFSZ	COUNT,F			; Test counter = 0 ?
	GOTO	SHIFT
	RETURN

; ERASE EEPROM  ;
ERASE	BSF	PORTA,3			; CS on
	CALL	EWEN				; Write enabled
	BCF	PORTA,3			; CS off
	NOP					; Delay 1us

	BSF	PORTA,3			; CS on
	MOVF	ADDRESS,W		; Send address
	MOVWF	DATA_L		
	BSF	DATA_L,7			; Set command
	BSF	DATA_L,6			; Erase command
	BSF	PORTA,1			; Send start bit
	BSF	PORTA,2			; Clock
	BCF	PORTA,2
	CALL	SENDBYTE	
	BCF	PORTA,3			; CS off
	NOP					; Delay 1us
			
	BSF	PORTA,3			; CS on
CHK_ERS	BTFSS	PORTA,0			; Erase complete? 
	GOTO	CHK_ERS			; No ! loop
	BCF	PORTA,3			; CS off
	NOP					; Delay 1us
	
	BSF	PORTA,3			; CS on
	CALL	EWDS				; Write disabled
	BCF	PORTA,3			; CS off
	NOP					; Delay 1us
	RETURN
