;**************************************************************************
; Program :     External interrupt
; Description : Demonstrate operation of external interrupt at RB0/INT pin
; Filename :    hlab1801.asm
;*************************************************************************

	LIST	P=16F628,W=-302

PCL	EQU	0x02		; Bank 0
PORTB	EQU	0x06	
INTCON	EQU	0x0B

OPTION_REG	EQU	0x81		; Bank 1
TRISB	EQU	0x86	

X1	EQU	0x70		; RAM
Y1	EQU	0x71
Z1	EQU	0x72
ROUND	EQU	0x73
BUFFER	EQU	0x74
X2	EQU	0x75
Y2	EQU	0x76

; BIT 
INTEDG	EQU	6			; For edge select
GIE	EQU	7			; Global interrupt bit
INTE	EQU	4			; RB0 interrupt enable bit
INTF	EQU	1			; RB0 interrupt flag

	__CONFIG	0x3F61	; XT-OSC,WDT-OFF,MCLR-ON

	ORG	0x000		; Reset vector
	GOTO	MAIN_PROG

	ORG	0x004		; Interrupt vector
ISR_PROG	BCF	INTCON,INTF	; Clear RB0 interrupt flag
	CALL	DELAY1
	RETFIE				; Return to main program
MAIN_PROG	
	BANKSEL	TRISB		; Select TRISB
	MOVLW	B'00000001' 	; RB0 as input only
	MOVWF	TRISB	
	BCF	OPTION_REG,INTEDG ; Interrupt on falling edge

	BANKSEL	PORTB		; Select PORTB
	CLRF	PORTB		; Clear PORTB

	BCF	INTCON,INTF	; Clear RB0 flag 	
	BSF	INTCON,INTE	; RB0 interrupt enable
	BSF	INTCON,GIE	; Enable all interrupt

REPEAT	MOVLW	.10
	MOVWF	ROUND
	CLRF	BUFFER

LOOP	MOVFW	BUFFER
	CALL	TABLE
	MOVWF	PORTB

	CALL	DELAY2
	INCF	BUFFER,1
	DECFSZ	ROUND,1
	GOTO	LOOP
	GOTO	REPEAT
	
TABLE	ADDWF	PCL,1
	DT	0DE,0FE,00E,0FA,0DA,0CC,09E,0B6,00C,07E
		;9 , 8 , 7 , 6 , 5 , 4 , 3 , 2 , 1 , 0

DELAY1	MOVLW	.5
	MOVWF	Z1
	CLRF	Y1
	CLRF	X1
	DECFSZ	X1,1
	GOTO	$-1
	DECFSZ	Y1,1
	GOTO	$-3
	DECFSZ	Z1,1
	GOTO	$-5
	RETURN
DELAY2	MOVLW	.35
	MOVWF	Y2
	CLRF	X2
	DECFSZ	X2,1
	GOTO	$-1
	DECFSZ	Y2,1
	GOTO	$-3
	RETURN
	END

