;******************************************************
; Program : RB-change interrupt
; Description : Demonstrate operation of RB7-RB4 logic changing interrupt
; Filename : hlab1802.asm
;******************************************************
	LIST	P=16F628,W=-302

PCL	EQU	0x02	; Bank 0
PORTA	EQU	0x05	
PORTB	EQU	0x06	
INTCON	EQU	0x0B
CMCON	EQU	0x1F

OPTION_REG	EQU	0x81	; Bank 1
TRISA	EQU	0x85
TRISB	EQU	0x86	

X	EQU	0x70	; RAM
Y	EQU	0x71
Z	EQU	0x72
BUFFER	EQU	0x73
; BIT 
RBPU	EQU	7	; For RB pull-up enable
GIE	EQU	7	; Global interrupt bit
RBIE	EQU	3	; RB7-RB4 interrupt enalble bit
RBIF	EQU	0	; RB7-RB4 change interrupt flag

	__CONFIG	0x3F61	; XT-OSC,WDT-OFF,MCLR-ON

	ORG	0x000	; Reset vector
	GOTO	MAIN_PROG

	ORG	0x004	; Interrupt vector
ISR_PROG	
	SWAPF	PORTB,0	; Find switch
	MOVWF	BUFFER
	COMF	BUFFER,0
	MOVWF	PORTA	; Show winner
	CALL	DELAY	
RELEASE7	BTFSS	PORTB,7
	GOTO	RELEASE7
RELEASE6	BTFSS	PORTB,6
	GOTO	RELEASE6
RELEASE5	BTFSS	PORTB,5
	GOTO	RELEASE5
RELEASE4	BTFSS	PORTB,4
	GOTO	RELEASE4
	MOVLW	0x0F
	MOVWF	PORTA	
	BCF	INTCON,RBIF	; Clear RB0 interrupt flag
	RETFIE		; Return to main program
MAIN_PROG	
	MOVLW	.7
	MOVWF	CMCON	; PORTA as digital I/O
	BANKSEL	TRISA	; Select TRISA
	MOVLW	B'11110000'
	MOVWF	TRISA
	BCF	OPTION_REG,RBPU ; Pull-up at port B

	BANKSEL	PORTA	; Select PORTA
	MOVLW	0x0F
	MOVWF	PORTA	
	CALL	DELAY	
	BCF	INTCON,RBIF	; Clear RBIF flag 	
	BSF	INTCON,RBIE	; RB change interrupt enable
	BSF	INTCON,GIE	; Enable all interrupt
LOOP	GOTO	LOOP

DELAY	MOVLW	.4	; Delay 0.8 second
	MOVWF	Z
	CLRF	Y
	CLRF	X
	DECFSZ	X,1
	GOTO	$-1
	DECFSZ	Y,1
	GOTO	$-3
	DECFSZ	Z,1
	GOTO	$-5
	RETURN
	END

