;***************************************************************************
; Program :		Switch matrix 4x3  
; Description : 	Demonstrate interfacing PIC16F628 with 4x3 matrix switches
; Filename :		hlab1901.asm
;***************************************************************************
	LIST	P=16F628,W=-302
PCL	EQU	0x02		; Bank 0
PORTA	EQU	0x05	
PORTB	EQU	0x06	
CMCON	EQU	0x1F	
TRISA	EQU	0x85		; Bank 1
TRISB	EQU	0x86	
COUNTER	EQU	0x70		; Common RAM
CHECK	EQU	0x71	

	__CONFIG	0x3F70
	ORG	0x000 
	
INIT_PORT 	MOVLW	.7
	MOVWF	CMCON		; Define RA3-RA0 to digital I/O
	BANKSEL	TRISA
	MOVLW	B'11110000'		; RA7,RA6,RA4 as input,RA3-RA0 as output
	MOVWF	TRISA
	CLRF	TRISB		; PORTB as output
	BANKSEL	PORTB
	CLRF	PORTB		; Turn-off display
START_SCAN	CLRF	COUNTER		
	MOVLW	B'11111110'		; Scan row 1
MOVWF	PORTA
LOOP_SCAN	BTFSS	PORTA,4		; Read column 1
	GOTO	FIND_SW0	

	BTFSS	PORTA,6		; Read column 2
	GOTO	FIND_SW1

	BTFSS	PORTA,7		; Read column 3
	GOTO	FIND_SW2
NEXT_ROW	SETC
	RLF	PORTA,1		; Scan next row
	MOVLW	.3
	ADDWF	COUNTER,1		; COUNTER = COUNTER + 3
	MOVLW	.12
	SUBWF	COUNTER,0		; w = F-w
	BZ	START_SCAN
	GOTO	LOOP_SCAN

FIND_SW2	INCF	COUNTER,1		; COUNTER = COUNTER + 1
FIND_SW1	INCF	COUNTER,1	
FIND_SW0	MOVFW	COUNTER
	CALL	DISPLAY
	MOVWF	PORTB
	
	GOTO	START_SCAN
	
RELEASE_SW_?	MOVFW	PORTA
	MOVWF	CHECK
	MOVLW	B'11010000'		; Mask bit
	ANDWF	CHECK,0
	SUBLW	B'11010000'
	BZ	START_SCAN		; Switch released
	GOTO	RELEASE_SW_?		; Switch not release

DISPLAY	ADDWF	PCL,1
	DT	006,05B,04F,066,06D,07D ;	CHAR.=> 1, 2, 3, 4, 5, 6
	DT	007,07F,06F,063,03F,049 ;	CHAR.=> 7, 8, 9, *, 0, #
	END

