;*************************************************************************
;Program :	LCD interface in 8-bit mode
;Description :	Demonstrate LCD interfacing and initializing by instruction
;Processor :	PIC16F628-04 MHz 
;filename :	hlab2401.asm
;*************************************************************************
 	LIST	P=16F628,W=-302

PORTA	EQU	0x05
PORTB	EQU	0x06
CMCON	EQU	0x1F
TRISA	EQU	0x85
TRISB	EQU	0x86
#DEFINE	DATA_BUS	PORTB	
#DEFINE	LOW_E	BCF PORTA,0
#DEFINE	HIGH_E	BSF PORTA,0
#DEFINE	LOW_RS  	BCF PORTA,1
#DEFINE	HIGH_RS	BSF PORTA,1

    	CBLOCK	0x70		 ; Assign variables
      	X,Y,Z
        	ENDC
 	__CONFIG	0x3F61	 ; OSC = XTAL 4 MHz. only
	ORG		0x0000
	CONSTANT	FUNCTION_VALUE  = B'00111000'
	CONSTANT	ENTRY_VALUE     = B'00000110'
	CONSTANT	DISPLAY_VALUE   = B'00001100'

ORIGIN:		MOVLW	.7	; Initialize port A
		MOVWF	CMCON	; RA3-RA0 as digital I/O

		CALL	DISABLE	; Disable after power-on reset	
		BANKSEL	TRISA	; Bank select
		MOVLW	B'11111100'
		MOVWF	TRISA	; RA1-RA0 as output
		CLRF	TRISB	; PORTB   as output
		BANKSEL	PORTB	; Bank select
		
		CALL	DELAY15M	; Wait for internal reset
		CALL	INITIAL_LCD ; Setup LCD
		CALL	CLEAR_SCREEN; Clear display

		MOVLW	0x80		; Write @address 80H
		CALL	SET_POSITION	; Set DDRAM position
		MOVLW	'I'
		CALL	WRITE_DATA
	MOVLW	'N'
	CALL	WRITE_DATA
	MOVLW	'E'
	CALL	WRITE_DATA
	MOVLW	'X'
	CALL	WRITE_DATA
BLINK:	CALL	DISPLAY_OFF	; Turn-off screen 
	CALL	DELAY400M	; Delay 400 ms
	CALL	DISPLAY_ON	; Turn-on screen
	CALL	DELAY400M	; Delay 400 ms
	GOTO	BLINK	

;**********************  SUBROUTINE  *********************
INITIAL_LCD:	CALL	FUNCTION_SET		; Initialize by instruction 
	CALL	FUNCTION_SET		; Very necessary if power 							; supply not stable 
	CALL	FUNCTION_SET		; 
	CALL	FUNCTION_SET		; True initial tart here
	CALL	ENTRY_MODE_SET
	CALL	DISPLAY_CONTROL
	RETURN
FUNCTION_SET:	MOVLW	FUNCTION_VALUE
	GOTO	WRITE_COMMAND2
ENTRY_MODE_SET:	MOVLW	ENTRY_VALUE
	GOTO	WRITE_COMMAND
DISPLAY_CONTROL:	MOVLW	DISPLAY_VALUE
	GOTO	WRITE_COMMAND
WRITE_DATA:	HIGH_RS
	GOTO	WRITE
WRITE_COMMAND:	LOW_RS
WRITE:	MOVWF	DATA_BUS
	NOP
	LOW_E				; Enable pulse
	CALL	DELAY40U
DISABLE:	HIGH_E
	RETURN
WRITE_COMMAND2:	LOW_RS
	MOVWF	DATA_BUS
	NOP
	LOW_E				; Enable pulse
	CALL	DELAY5M
	HIGH_E
	RETURN

CLEAR_SCREEN:	MOVLW	B'00000001'	
	GOTO	WRITE_COMMAND2

RETURN_HOME:	MOVLW	B'00000010' 		; Return home
	GOTO	WRITE_COMMAND2

DISPLAY_OFF:	MOVLW	B'00001000'
	GOTO	WRITE_COMMAND	

DISPLAY_ON:	MOVLW	B'00001100'
	GOTO	WRITE_COMMAND

SHIFT_LEFT:	MOVLW	B'00011000'
	GOTO	WRITE_COMMAND

SHIFT_RIGHT:	MOVLW	B'00011111'
	GOTO	WRITE_COMMAND

CURSOR_LEFT:	MOVLW	B'00010000'
	GOTO	WRITE_COMMAND

CURSOR_RIGHT:	MOVLW	B'00010100'
	GOTO	WRITE_COMMAND

SET_POSITION:	GOTO	WRITE_COMMAND
	
DELAY5M:	MOVLW	.7		; Delay ~ 5.4 ms
	GOTO	DELAY	
DELAY15M:	MOVLW	.20		; Delay ~ 15 ms for initial reset
DELAY:	MOVWF	X			
	CLRF	Y
	DECFSZ	Y,1
	GOTO	$-1
	DECFSZ	X,1
	GOTO	$-3
	RETURN
DELAY40U:	MOVLW	.12		; Delay ~ 40 us
	MOVWF	X
	DECFSZ	X,1
	GOTO	$-1
	RETURN
DELAY400M:	MOVLW	.2		; Delay ~400 ms
	MOVWF	Z
	CLRF	Y
	CLRF	X
	DECFSZ	X,1
	GOTO	$-1
	DECFSZ	Y,1
	GOTO	$-3
	DECFSZ	Z,1
	GOTO	$-5
	RETURN

	
	END
