;*************************************************************************
;Program :	Message showing
;Description :	Demonstrate showing message on LCD in 8-bit mode
;Processor :	PIC16F628-04 MHz 
;filename :	hlab2403.asm
;*************************************************************************
	LIST	P=16F628,W=-302

PCL	EQU	0x02			; Bank 0
PORTA	EQU	0x05	
PORTB	EQU	0x06
CMCON	EQU	0x1F
TRISA	EQU	0x85	 		; Bank 1
TRISB	EQU	0x86
#DEFINE	DATA_BUS	PORTB	
#DEFINE	LOW_E	BCF PORTA,0
#DEFINE	HIGH_E	BSF PORTA,0
#DEFINE	LOW_RS  	BCF PORTA,1
#DEFINE	HIGH_RS	BSF PORTA,1

	CBLOCK	0x70			; Common RAM
      	X,Y,Z,COUNTER1,COUNTER2
        	ENDC

          	__CONFIG	0x3F61		; XT mode 4 MHz.

	ORG	0x0000
	CONSTANT	FUNCTION_VALUE  = B'00111000'
	CONSTANT	ENTRY_VALUE     = B'00000110'
	CONSTANT	DISPLAY_VALUE   = B'00001100'

START:	MOVLW	.7		; Initialize port A
	MOVWF	CMCON		; RA3-RA0 as digital I/O

	CALL	DISABLE		; Disable after power-on reset
	BANKSEL	TRISA		; Bank select
	MOVLW	B'11111100'
	MOVWF	TRISA		; RA1-RA0 as output
	CLRF	TRISB		; Port B as output
	BANKSEL	PORTB		; Bank select
	
	CALL	DELAY15M		; Wait for internal reset
	CALL	INITIAL_LCD 		; Setup LCD
	CALL	CLEAR_SCREEN		; Clear display
	
;************************  WRITE DATA ON SCREEN  **************************
	MOVLW	0x80		; Write @address 80H
	CALL	SET_POSITION		; Set DDRAM position

DO_AGAIN:	CALL	CLEAR_SCREEN
	MOVLW	.12
	MOVWF	COUNTER2   		; Counter
	MOVLW	.00		; Offset value
	CALL	WRITE_CHAR1
	CALL	LOOP_1

	MOVLW	.11		; Delay 2s
	CALL	DELAY
	CALL	CLEAR_SCREEN

	MOVLW	0x91
	CALL	SET_POSITION
	MOVLW	.28
	MOVWF	COUNTER2
	MOVLW	.13
	CALL	WRITE_CHAR1
	
	MOVLW	.17
	MOVWF	COUNTER2
LOOP3:	CALL	SHIFT_LEFT
	MOVLW	.1		; Delay 200 ms
	CALL	DELAY
	DECF	COUNTER2,1
	SKPZ			; Skip if complete
	GOTO	LOOP3

	MOVLW	0xD1		; Write @address D1H
	CALL	SET_POSITION
	MOVLW	.42
	MOVWF	COUNTER2
	MOVLW	.29
	CALL	WRITE_CHAR2
	GOTO	DO_AGAIN		; Do forever

WRITE_CHAR1:	MOVWF	COUNTER1		; COUNTER1 = offset
WRITE_LOOP1:	CALL	GET_CHAR		; Call table
	CALL	WRITE_DATA  		; Write characters to screen
	INCF	COUNTER1,1		; COUNTER1 = COUNTER1+1
	MOVFW	COUNTER2
	SUBWF	COUNTER1,0  		; Write complete ?
	SKPZ			; Yes, skip (Z=1)
	GOTO	WRITE_LOOP1 		; No, do again
	RETURN
WRITE_CHAR2:	MOVWF	COUNTER1		; COUNTER1 = offset
WRITE_LOOP2:	CALL	GET_CHAR		; Call table
	CALL	WRITE_DATA  		; Write characters to screen
	MOVLW	.3		; Delay 600 ms
	CALL	DELAY
	INCF	COUNTER1,1		; COUNTER1 = COUNTER1+1
	MOVFW	COUNTER2
	SUBWF	COUNTER1,0  		; Write complete ?
	SKPZ			; Yes, skip (Z=1)
	GOTO	WRITE_LOOP2 		; No, do again
	MOVLW	.11		; Delay 2 s
	CALL	DELAY
	RETURN	
LOOP_1:	CLRF	COUNTER1
LOOP_2:	MOVLW	.2		; Delay time value
	CALL	DELAY	
	CALL	DISPLAY_OFF
	MOVLW	.2		; Delay time value
	CALL	DELAY
	CALL	DISPLAY_ON
	INCF	COUNTER1,1		; COUNTER1 = COUNTER1+1
	BTFSS	COUNTER1,2  		; Flashing 4 times
	GOTO	LOOP_2
	RETURN
GET_CHAR:	MOVFW	COUNTER1		; w = COUNTER1
	ADDWF	PCL,1
	DT	"I'm PICMICRO I DO EVERYTHING AS YOUR SLAVE"

;****************************  SUBROUTINE  ********************************
INITIAL_LCD:	CALL	FUNCTION_SET		; Initialize MCU
	CALL	FUNCTION_SET	
	CALL	FUNCTION_SET	
	CALL	FUNCTION_SET	
	CALL	ENTRY_MODE_SET
	CALL	DISPLAY_CONTROL
	RETURN

FUNCTION_SET:	MOVLW	FUNCTION_VALUE
	GOTO	WRITE_COMMAND2
ENTRY_MODE_SET:	MOVLW	ENTRY_VALUE
	GOTO	WRITE_COMMAND
DISPLAY_CONTROL:	MOVLW	DISPLAY_VALUE
	GOTO	WRITE_COMMAND
WRITE_DATA:	HIGH_RS
	GOTO	WRITE
WRITE_COMMAND:	LOW_RS
WRITE:	MOVWF	DATA_BUS
	NOP
	LOW_E		; Enable pulse
	CALL	DELAY40U

DISABLE:	HIGH_E
	RETURN

WRITE_COMMAND2:	LOW_RS
	MOVWF	DATA_BUS
	NOP
	LOW_E		; Enable pulse
	CALL	DELAY5M
	HIGH_E
	RETURN

CLEAR_SCREEN:	MOVLW	B'00000001'	
	GOTO	WRITE_COMMAND2
RETURN_HOME:	MOVLW	B'00000010' ; Return home
	GOTO	WRITE_COMMAND2
DISPLAY_OFF:	MOVLW	B'00001000'
	GOTO	WRITE_COMMAND
DISPLAY_ON:	MOVLW	B'00001100'
	GOTO	WRITE_COMMAND
SHIFT_LEFT:	MOVLW	B'00011000'
	GOTO	WRITE_COMMAND
SHIFT_RIGHT:	MOVLW	B'00011111'
	GOTO	WRITE_COMMAND
CURSOR_LEFT:	MOVLW	B'00010000'
	GOTO	WRITE_COMMAND
CURSOR_RIGHT:	MOVLW	B'00010100'
	GOTO	WRITE_COMMAND
SET_POSITION:	GOTO	WRITE_COMMAND

;*************************  DELAY ROUTINE  ********************************
DELAY5M:	MOVLW	.7	; Delay ~ 5.4 ms
	GOTO	DELAY_LINE	
DELAY15M:	MOVLW	.20	; Delay ~ 15 ms for internal reset
DELAY_LINE:	MOVWF	X	
	CLRF	Y
	DECFSZ	Y,1
	GOTO	$-1
	DECFSZ	X,1
	GOTO	$-3
	RETURN

DELAY40U:	MOVLW	.12	; Delay ~ 40 us
	MOVWF	X
	DECFSZ	X,1
	GOTO	$-1
	RETURN

DELAY:	MOVWF	Z
	CLRF	Y
	CLRF	X
	DECFSZ	X,1
	GOTO	$-1
	DECFSZ	Y,1
	GOTO	$-3
	DECFSZ	Z,1
	GOTO	$-5
	RETURN

	END
