;*************************************************************************
;Program :	LCD game
;Description :	Demonstrate showing message on LCD in 4-bit mode
;Processor :	PIC16F628-04 MHz 
;filename :	hlab2404.asm
;*************************************************************************
	LIST	P=16F628,W=-302
PCL	EQU	0x02		; BANK0
PORTB	EQU	0x06
TRISB	EQU	0x86		; BANK1
     	CBLOCK 0x70		; RAM
       	V,W,X,Y,Z,BUFFER0,BUFFER1,R0,R1,NUM_CHAR,OFFSET,DURATION
        	ENDC 

#DEFINE	DATA_BUS	PORTB	
#DEFINE	LOW_E	BCF PORTB,2
#DEFINE	HIGH_E	BSF PORTB,2
#DEFINE	LOW_RS	BCF PORTB,3
#DEFINE	HIGH_RS	BSF PORTB,3

	__CONFIG	0x3F61				; XT-MODE : 4 MHz.
	ORG	0x0000
;***************************  MAIN  **************************************
    CONSTANT	FUNCTION_VALUE  = B'00101000'	; LCD command
    CONSTANT	ENTRY_VALUE     = B'00000110'	; LCD command
    CONSTANT	DISPLAY_VALUE   = B'00001100'	; LCD command
INIT_PORT:      	CALL	DISABLE				; Disable power-on reset	
	BANKSEL	TRISB				; Bank select
	MOVLW	B'00000011'
	MOVWF	TRISB				; RB7-RB2 as output
	BANKSEL	PORTB				; Bank select
                	CALL	DELAY15M				; Wait for internal reset
	CALL	INITIAL_LCD 			; Setup LCD
	CALL	CLEAR_SCREEN			; Clear display
FIRST:	CALL	SOUND				; First job : generate sound
	MOVLW	0x80				; Write @address H80'
	CALL	SET_POSITION    			; Set DDRAM address
	MOVLW	.9				; Number of character
	MOVWF	NUM_CHAR
	CLRF	OFFSET				; Address of character in table
	CALL	WRITE_CHAR
	MOVLW	.10
	CALL	DELAY				; Delay 2 second
SECOND:	CALL	CLEAR_SCREEN			; Second job : clear screen
	MOVLW	.14				; 14 characters
	MOVWF	NUM_CHAR	
	MOVLW	.9				; Offset value
	MOVWF	OFFSET
	CALL	WRITE_CHAR			; Write chatacter
	CALL	PRESS_ENTER			; Show message on line 2
THIRD:						; Third job : reading switch
READ_SW1:	BTFSC	PORTB,0				; Is switch press ?	
	GOTO	READ_SW1				; No, read again
PLAY_AGAIN:	CALL	CLEAR_SCREEN			; Yes, do next job
	MOVLW	.14				; 14 characters
	MOVWF	NUM_CHAR
	MOVLW	.34				; address 34 in table
	MOVWF	OFFSET
	CALL	WRITE_CHAR			; Write character
	CALL	SOUND				; Repeat 5 times
	CALL	SOUND		
	CALL	SOUND	
	CALL	SOUND	
	CALL	SOUND	
FOURTH:	CALL	RANDOM				; Fourth job
	MOVWF	V	 
	MOVWF	W	
	MOVLW	B'11110000'	
	ANDWF	V,1				; Mask bit
	SWAPF	V,0				; Swap file
	CALL	FIND_CHAR			; Call table
	MOVWF	V				; Store ASCII value
	MOVLW	B'00001111'
	ANDWF	W,0				; Mask bit
	CALL	FIND_CHAR			; Call table
	MOVWF	W				; Store ASCII value
FIFTH:	CALL	CLEAR_SCREEN			; Fifth job
	MOVLW	.7
	MOVWF	NUM_CHAR
	MOVLW	.9
	MOVWF	OFFSET
	CALL	WRITE_CHAR
	MOVLW	'='
	CALL	WRITE_DATA
	MOVLW	' '
	CALL	WRITE_DATA
	MOVFW	V
	CALL	WRITE_DATA
	MOVFW	W
	CALL	WRITE_DATA
	MOVLW	.5
	CALL	DELAY
	CALL	PRESS_ENTER
WAIT_RELEASE	BTFSS	PORTB,0				; Is switch press? 	
	GOTO	WAIT_RELEASE			; No, read again
READ_SW2	BTFSC	PORTB,0				; Yes, do next job
	GOTO	READ_SW2
	GOTO	PLAY_AGAIN
	
WRITE_CHAR:	CALL	CHAR_GEN				; Call table
	CALL	WRITE_DATA      			; Write character
	INCF	OFFSET,1				; Increase offset
	DECF	NUM_CHAR,1			; Decrease counter 
	SKPZ					; Yes, skip (Z=1)
	GOTO	WRITE_CHAR      			; No, do again 
	RETURN
CHAR_GEN:	MOVFW	OFFSET
	ADDWF	PCL,1
TABLE1:	DT	"LET'S FUN#RANDOM  ReadyPress ENTER"
TABLE2:	DT	"#RANDOMMING..."	
FIND_CHAR:	ADDWF	PCL,1
TABLE3:	DT	"0123456789ABCDEF"	
PRESS_ENTER:	MOVLW	0xC0				; Write @address HC0'
	CALL	SET_POSITION			; Set DDRAM address
	MOVLW	.11
	MOVWF	NUM_CHAR
	MOVLW	.23
	MOVWF	OFFSET
	CALL	WRITE_CHAR
	RETURN

;***************************  SUBROUTINES  *******************************
INITIAL_LCD:	CALL	FUNCTION_INIT			; Initialze by instruction 
	CALL	FUNCTION_SET			; True initial start here
	CALL	ENTRY_MODE_SET
	CALL	DISPLAY_CONTROL
	RETURN

FUNCTION_INIT:	MOVLW	0x33				; First & second write
	CALL	WRITE_COMMAND2
	MOVLW	0x32				; Third write & true initial
	GOTO	WRITE_COMMAND
FUNCTION_SET:	MOVLW	FUNCTION_VALUE
	GOTO	WRITE_COMMAND2
ENTRY_MODE_SET:	MOVLW	ENTRY_VALUE
	GOTO	WRITE_COMMAND
DISPLAY_CONTROL:	MOVLW	DISPLAY_VALUE
	GOTO	WRITE_COMMAND
WRITE_DATA:	MOVWF	BUFFER0				; Write high nibble
	CALL	MASK_BIT_DATA
	CALL	DELAY40U				; Delay 40 us
	HIGH_E
	SWAPF	BUFFER0,0			; Write low nibble
	CALL	MASK_BIT_DATA
	CALL	DELAY40U				; Delay 40 us
DISABLE:	HIGH_E
	RETURN

WRITE_COMMAND:	MOVWF	BUFFER0				; Write high nibble
	CALL	MASK_BIT_CMD
	CALL	DELAY40U				; Delay 40 us
	HIGH_E
	SWAPF	BUFFER0,0			; Write low nibble
	CALL	MASK_BIT_CMD
	CALL	DELAY40U				; Delay 40 us
	HIGH_E
	RETURN
WRITE_COMMAND2:	MOVWF	BUFFER0				; Write high nibble
	CALL	MASK_BIT_CMD
	CALL	DELAY5M				; Delay 5 ms
	HIGH_E
	SWAPF	BUFFER0,0			; Write low nibble
	CALL	MASK_BIT_CMD
	CALL	DELAY5M				; Delay 5 ms
	HIGH_E
	RETURN
MASK_BIT_CMD:	MOVWF	BUFFER1
	LOW_RS
	BCF	BUFFER1,3			; For RS = 0
	BSF	BUFFER1,2			; For E  = 1
	MOVFW	BUFFER1
	MOVWF	DATA_BUS
	NOP
	LOW_E					; Enable pulse
	RETURN

MASK_BIT_DATA:	MOVWF	BUFFER1
	HIGH_RS
	BSF	BUFFER1,3			; FOR RS  = 1
	BSF	BUFFER1,2			; FOR E   = 1
	MOVFW	BUFFER1
	MOVWF	DATA_BUS
	NOP
	LOW_E					; Enable pulse
	RETURN		

CLEAR_SCREEN:	MOVLW	B'00000001'	
	GOTO	WRITE_COMMAND2
RETURN_HOME:	MOVLW	B'00000010' 			; Return home
	GOTO	WRITE_COMMAND2
DISPLAY_OFF:	MOVLW	B'00001000' 			; Turn-off display
	GOTO	WRITE_COMMAND
DISPLAY_ON:	MOVLW	B'00001100' 			; Turn-on display
	GOTO	WRITE_COMMAND
SET_POSITION:	GOTO	WRITE_COMMAND

RANDOM:	MOVFW	R1
	IORWF	R0,0
	SKPNZ
	INCF	R0,1
	MOVLW	0x80
	BTFSC	R0,4
	XORWF	R1,1
	BTFSC	R0,2
	XORWF	R1,1
	BTFSC	R0,1
	XORWF	R1,1
	RLF	R1,0
	RLF	R0,1
	RLF	R1,1
	MOVFW	R0
	RETURN	

DELAY5M:	MOVLW	.7			; Delay ~ 5.4 ms
	GOTO	DELAYLINE	
DELAY15M:	MOVLW	.20			; Delay ~ 15 ms for internal reset
DELAYLINE:	MOVWF	X	
	CLRF	Y
	DECFSZ	Y,1
	GOTO	$-1
	DECFSZ	X,1
	GOTO	$-3
	RETURN

DELAY40U:	MOVLW	.12			; Delay ~ 40 us
	MOVWF	X
	DECFSZ	X,1
	GOTO	$-1
	RETURN
DELAY:	MOVWF	Z
	CLRF	Y
	CLRF	X
	DECFSZ	X,1
	GOTO	$-1
	DECFSZ	Y,1
	GOTO	$-3
	DECFSZ	Z,1
	GOTO	$-5
	RETURN

SOUND:	MOVLW	.40			; Duration
	MOVWF	DURATION
	BANKSEL	TRISB			; Select bank 1
	BCF	TRISB,1			; RB1 as output
	BANKSEL	PORTB			; Return to bank 0
SOUND_LOOP:	BSF	PORTB,1
	MOVLW	.1			; Frequency
CALL	SOUND_DELAY
	BCF	PORTB,1
	MOVLW	.1			; Frequency
	CALL	SOUND_DELAY
	DECFSZ	DURATION,1	
	GOTO	SOUND_LOOP
	MOVLW	.123
	CALL	SOUND_DELAY		; Stop
	BANKSEL	TRISB			; Select bank 1
 	BSF	TRISB,1			; Return to input
	BANKSEL	PORTB			; Return to bank 0
	RETURN
SOUND_DELAY:	CLRF	X
	MOVWF	Y
	DECFSZ	X,1
	GOTO	$-1
	DECFSZ	Y,1
	GOTO	$-3
	RETURN

	END
