;*************************************************************************
;Program :	Capture operation in CCP module
;Description :	Measure period by interrupt in capture mode of CCP module
;Processor :	PIC16F628-04 MHz 
;filename :	hlab2501.asm
;*************************************************************************
	LIST	P=16F628,W=-302
	INCLUDE	P16F628.INC		; Use Microchip include file
	__CONFIG	0x3F61			; Use 4MHz XT-mode
;*************************************************************************
    	CBLOCK	0x20			; Variable bank 0
     	COUNTER,H_BYTE,L_BYTE,R0,R1,R2,R3,R4,X,Y,TEMP_DATA
      	ENDC

    	CBLOCK 	0x30
     	ASCII1,ASCII2,ASCII3,ASCII4,ASCII5,TEMP ; ASCII var. bank 0
      	ENDC 

#DEFINE	DATA_BUS	PORTB			; 4 bit data bus
#DEFINE	LOW_E	BCF PORTA,0		; Enable pin
#DEFINE	HIGH_E	BSF PORTA,0
#DEFINE	LOW_RS  	BCF PORTA,1		; RS pin
#DEFINE	HIGH_RS	BSF PORTA,1
;*************************************************************************
	CONSTANT	FUNCTION_VALUE  = B'00101000'	; LCD setup
	CONSTANT	ENTRY_VALUE     = B'00000110'	; LCD setup
	CONSTANT	DISPLAY_VALUE   = B'00001100'	; LCD setup

	ORG	0x000			; Reset vector
	GOTO	MAIN_PROGRAM	
	ORG	0x004			; Interrupt vector
	GOTO	INTERRUPT
MAIN_PROGRAM:	MOVLW	.7	
	MOVWF	CMCON		; RA3-RA0 as digital pin
	CALL	DISABLE		; Disable after power-on reset	
	BANKSEL	TRISA		; Bank select
	MOVLW	B'11111100'
	MOVWF	TRISA		; RA1-RA0 as output
	MOVLW	B'00001111'
	MOVWF	TRISB		; RB7-RB4 as output
	BANKSEL	PORTB		; Bank select
	
	CALL	DELAY15M		; For LCD internal reset
	CALL	INITIAL_LCD 	; Setup LCD
	CALL	CLEAR_SCREEN	; Clear display
CAPTURE_MODE:	
	BANKSEL	PIE1		; Bank 1
	BSF	PIE1,CCP1IE 	; Capture interrupt
	BANKSEL	T1CON		; Return to bank 0	
	MOVLW	B'00000000' 	; Internal clock,
	MOVWF	T1CON       		; TIMER1 prescaler = 1:1
	CALL	START
	BSF	INTCON,PEIE	; Peripheral interrupt		
	GOTO	$		; Wait for interrupt
INTERRUPT:	CLRF	CCP1CON		; Turn-off capture module
	BCF	T1CON,TMR1ON	; Turn-off TIMER1	

	CALL	CALCULATE   	; Do the math, F = 1/T

START:	CLRF	TMR1H		; Clear TIMER1
	CLRF	TMR1L
	BCF	PIR1,CCP1IF	; Clear flag  before turn-on
	MOVLW	B'00000101' 	
	MOVWF	CCP1CON
	BCF	PIR1,CCP1IF	; Clear flag before test 
	BTFSS	PIR1,CCP1IF	; Teat flag for starting capture
	GOTO	$-1		; Wait for rising edge
	BSF	T1CON,TMR1ON	; Capture again
	BCF	PIR1,CCP1IF
	RETFIE			; GIE = 1 after this instruction	
CALCULATE:	MOVFW	CCPR1H		; Period high byte
	MOVWF	H_BYTE
	MOVFW	CCPR1L		; Period low byte
	MOVWF	L_BYTE
	CALL	BCD16		; Convert binary to BCD
	CALL	ASCIIADJ		; Convert BCD to ASCII
	CALL	SHOW_PERIOD	; Show period on LCD
	MOVLW	.255		; Delay for messages on LCD
	CALL	DELAY
	RETURN			
SHOW_PERIOD:	CALL	RETURN_HOME	; Messages
	MOVLW	'P'
	CALL	WRITE_DATA
	MOVLW	'E'
	CALL	WRITE_DATA
	MOVLW	'R'
	CALL	WRITE_DATA
	MOVLW	'I'
	CALL	WRITE_DATA
	MOVLW	'O'
	CALL	WRITE_DATA
	MOVLW	'D'
	CALL	WRITE_DATA
	MOVLW	'='
	CALL	WRITE_DATA
	MOVLW	' '
	CALL	WRITE_DATA

	MOVFW	ASCII1	
	CALL	WRITE_DATA
	MOVFW	ASCII2
	CALL	WRITE_DATA
	MOVFW	ASCII3
	CALL	WRITE_DATA
	MOVFW	ASCII4
	CALL	WRITE_DATA
	MOVFW	ASCII5
	CALL	WRITE_DATA

	MOVLW	' '
	CALL	WRITE_DATA
	MOVLW	'u'
	CALL	WRITE_DATA
	MOVLW	'S'
	CALL	WRITE_DATA

	RETURN
;******************************  SUBROUTINE   ****************************
; Adjust BCD to ASCII
;*************************
ASCIIADJ:	MOVLW	0x30	
 	ADDWF	R0,0
	MOVWF	ASCII1
	SWAPF	R1,0
	MOVWF	TEMP
	ANDLW	0x0F
	ADDLW	0x30
	MOVWF	ASCII2
	SWAPF	TEMP,0
	ANDLW	0x0F
	ADDLW	0x30
	MOVWF	ASCII3

	SWAPF	R2,0
	MOVWF	TEMP
	ANDLW	0x0F
	ADDLW	0x30
	MOVWF	ASCII4
	SWAPF	TEMP,0
	ANDLW	0x0F
	ADDLW	0x30
	MOVWF	ASCII5
	RETURN
	
; ADJUST BINARY TO BCD
;***************************
BCD16:	CLRC
	MOVLW   .16
	MOVWF   COUNTER
	CLRF    R2
	CLRF    R1
	CLRF    R0
LOOP16:	RLF     L_BYTE,1
	RLF     H_BYTE,1
	RLF     R2,1
	RLF     R1,1
	RLF     R0,1
	DECFSZ  COUNTER,1
	GOTO    ADJDEC
	RETURN	
	
ADJDEC:	MOVLW   R2
	MOVWF   FSR
	CALL    ADJBCD
	MOVLW   R1
	MOVWF   FSR
	CALL    ADJBCD
	MOVLW   R0
	MOVWF   FSR
	CALL    ADJBCD
	GOTO    LOOP16
ADJBCD:        MOVLW   .3
	ADDWF   INDF,0
	MOVWF   TEMP
	BTFSC   TEMP,3          
	MOVWF   INDF
	MOVLW   0x30
	ADDWF   INDF,0
	MOVWF   TEMP
	BTFSC   TEMP,7          
	MOVWF   INDF               
	RETURN

; LCD subroutine
;***********************
INITIAL_LCD:	CALL	FUNCTION_INIT		; Initialze by instruction 
	CALL	FUNCTION_SET		; True initial start here
	CALL	ENTRY_MODE_SET
	CALL	DISPLAY_CONTROL
	RETURN

FUNCTION_INIT:	MOVLW	0x33			; First & second write
	CALL	WRITE_COMMAND2
	MOVLW	0x32			; Third write & true initial
	GOTO	WRITE_COMMAND

FUNCTION_SET:	MOVLW	FUNCTION_VALUE
	GOTO	WRITE_COMMAND2

ENTRY_MODE_SET:	MOVLW	ENTRY_VALUE
	GOTO	WRITE_COMMAND

DISPLAY_CONTROL:	MOVLW	DISPLAY_VALUE
	GOTO	WRITE_COMMAND

WRITE_DATA:	HIGH_RS
	GOTO	WRITE
WRITE_COMMAND:	LOW_RS
WRITE:	MOVWF	DATA_BUS
	MOVWF	TEMP_DATA
	LOW_E				; Enable pulse
	CALL	DELAY40U
	HIGH_E
	SWAPF	TEMP_DATA,0
	MOVWF	DATA_BUS
	NOP
	LOW_E
	CALL	DELAY40U
DISABLE:	HIGH_E
	RETURN

WRITE_COMMAND2:	LOW_RS
	MOVWF	DATA_BUS
	MOVWF	TEMP_DATA
	LOW_E				; Enable pulse
	CALL	DELAY5M
	HIGH_E
	SWAPF	TEMP_DATA,0
	MOVWF	DATA_BUS
	NOP
	LOW_E
	CALL	DELAY5M
	HIGH_E
	RETURN

CLEAR_SCREEN:	MOVLW	B'00000001'	
	GOTO	WRITE_COMMAND2

RETURN_HOME:	MOVLW	B'00000010' 		; Return home
	GOTO	WRITE_COMMAND2

DELAY5M:	MOVLW	.7			; Delay ~ 5.4 ms
	GOTO	DELAY	
DELAY15M:	MOVLW	.20			; Delay ~ 15 ms for internal reset
DELAY:	MOVWF	X
	CLRF	Y
	DECFSZ	Y,1
	GOTO	$-1
	DECFSZ	X,1
	GOTO	$-3
	RETURN

DELAY40U:	MOVLW	.12			; Delay ~ 40 us
	MOVWF	X
	DECFSZ	X,1
	GOTO	$-1
	RETURN
	
	END
