;********************************************************************
;Program :	Frequency counter using CCP1 module
;Description :	Measure frequency in capture mode of CCP module
;Processor :	PIC16F628-04 MHz 
;filename :	hlab2502.asm
;********************************************************************
	LIST	P=16F628,W=-302
	INCLUDE	P16F628.INC	; Use Microchip include file
	__CONFIG	0x3F61	; Use 4 MHz XT-mode
    	CBLOCK	0x20	; DIV variable bank 0 
     	TEMP,AARGB0,AARGB1,AARGB2,BARGB0,BARGB1,REMB0,REMB1,REMB2,LOOPCOUNT
      	ENDC

    	CBLOCK	0x30	; BCD variable bank 0
     	COUNTER,H_BYTE,L_BYTE,R0,R1,R2,R3,R4,X,Y,TEMP_DATA
      	ENDC

	CBLOCK 	0x40
     	ASCII1,ASCII2,ASCII3,ASCII4,ASCII5 	; ASCII variable bank 0
      	ENDC 

#DEFINE	DATA_BUS	PORTB		; 4 bit data bus
#DEFINE	LOW_E	BCF PORTA,0	; Enable pin
#DEFINE	HIGH_E	BSF PORTA,0
#DEFINE	LOW_RS	BCF PORTA,1	; RS pin
#DEFINE	HIGH_RS	BSF PORTA,1
;*************************************************************************
	CONSTANT	FUNCTION_VALUE  = B'00101000'	; LCD setup
	CONSTANT	ENTRY_VALUE     = B'00000110'	; LCD setup
	CONSTANT	DISPLAY_VALUE   = B'00001100'	; LCD setup

	ORG	0x000		; Reset vector
	GOTO	MAIN_PROGRAM	
	ORG	0x004		; Interrupt vector
	GOTO	INTERRUPT
MAIN_PROGRAM	MOVLW	.7	
	MOVWF	CMCON		; RA3-RA0 as digital I/O
	CALL	DISABLE		; Disable after power-on reset	
	BANKSEL	TRISA		; Bank select
	MOVLW	B'11111100'
	MOVWF	TRISA		; RA1-RA0 as output
	MOVLW	B'00001111'
	MOVWF	TRISB		; RB7-RB4 as output
	BANKSEL	PORTB		; Bank select
	CALL	DELAY15M		; For LCD internal reset
	CALL	INITIAL_LCD 	; Setup LCD
	CALL	CLEAR_SCREEN	; Clear display
CAPTURE_MODE	
	BANKSEL	PIE1		; Bank 1
	BSF	PIE1,CCP1IE 	; Capture interrupt
	BANKSEL	T1CON		; Return to bank 0	
	MOVLW	B'00000000' 	; Internal clock, 
	MOVWF	T1CON       		; Timer1 prescaler = 1:1
	CALL	START
	BSF	INTCON,PEIE	; Peripheral interrupt		
	GOTO	$		; Wait for interrupt
INTERRUPT	CLRF	CCP1CON			; Turn-off capture module
	BCF	T1CON,TMR1ON		; Turn-off Timer1	
	CALL	CALCULATE   			; Do the math, F = 1/T
START	CLRF	TMR1H			; Clear Timer1
	CLRF	TMR1L
	BCF	PIR1,CCP1IF		; Clear flag before turn-on
	MOVLW	B'00000101' 	
	MOVWF	CCP1CON
	BCF	PIR1,CCP1IF		; Clear flag before test
	BTFSS	PIR1,CCP1IF		; Test flag for starting measure
	GOTO	$-1			; Wait for rising edge
	BSF	T1CON,TMR1ON		; Measure again
	BCF	PIR1,CCP1IF
	RETFIE				; GIE = 1 after this instuction

CALCULATE	MOVFW	CCPR1H			; Period high byte
	MOVWF	BARGB0
	MOVFW	CCPR1L			; Period low byte
	MOVWF	BARGB1
	CALL	DIV2416			; Divide 24/16 unsign

	MOVFW	AARGB1			; F = 1000000/period Hz
	MOVWF	H_BYTE
	MOVFW	AARGB2
	MOVWF	L_BYTE
	CALL	BCD16			; Convert binary to BCD
	CALL	ASCIIADJ			; Convert BCD to ASCII
	CALL	SHOW_FREQ			; Show frequency on LCD
	MOVLW	.255			; Delay for messaging on LCD
	CALL	DELAY
	RETURN			
SHOW_FREQ	CALL	RETURN_HOME
	MOVLW	'F'
	CALL	WRITE_DATA
	MOVLW	'R'
	CALL	WRITE_DATA
	MOVLW	'E'
	CALL	WRITE_DATA
	MOVLW	'Q'
	CALL	WRITE_DATA
	MOVLW	'='
	CALL	WRITE_DATA
	MOVLW	' '
	CALL	WRITE_DATA

	MOVFW	ASCII1			; Frequency value  	
	CALL	WRITE_DATA
	MOVFW	ASCII2
	CALL	WRITE_DATA
	MOVFW	ASCII3
	CALL	WRITE_DATA
	MOVFW	ASCII4
	CALL	WRITE_DATA
	MOVFW	ASCII5
	CALL	WRITE_DATA

	MOVLW	' '
	CALL	WRITE_DATA
	MOVLW	'H'
	CALL	WRITE_DATA
	MOVLW	'z'
	CALL	WRITE_DATA
	RETURN
;******************************  SUBROUTINE   ****************************
ASCIIADJ	MOVLW	0x30				; Adjust BCD to ASCII
 	ADDWF	R0,0
	MOVWF	ASCII1
	SWAPF	R1,0
	MOVWF	TEMP
	ANDLW	0x0F
	ADDLW	0x30
	MOVWF	ASCII2
	SWAPF	TEMP,0
	ANDLW	0x0F
	ADDLW	0x30
	MOVWF	ASCII3
	SWAPF	R2,0
	MOVWF	TEMP
	ANDLW	0x0F
	ADDLW	0x30
	MOVWF	ASCII4
	SWAPF	TEMP,0
	ANDLW	0x0F
	ADDLW	0x30
	MOVWF	ASCII5
	RETURN	
BCD16:	CLRC					; Adjust binary to BCD
	MOVLW   .16
	MOVWF   COUNTER
	CLRF    R2
	CLRF    R1
	CLRF    R0
LOOP16 	RLF     L_BYTE,1
	RLF     H_BYTE,1
	RLF     R2,1
	RLF     R1,1
	RLF     R0,1
	DECFSZ  COUNTER,1
	GOTO    ADJDEC
	RETURN	
	
ADJDEC	MOVLW   R2
	MOVWF   FSR
	CALL    ADJBCD
	MOVLW   R1
	MOVWF   FSR
	CALL    ADJBCD
	MOVLW   R0
	MOVWF   FSR
	CALL    ADJBCD
	GOTO    LOOP16
ADJBCD:	MOVLW   .3
	ADDWF   INDF,0
	MOVWF   TEMP
	BTFSC   TEMP,3          
	MOVWF   INDF
	MOVLW   0x30
	ADDWF   INDF,0
	MOVWF   TEMP
	BTFSC   TEMP,7          
	MOVWF   INDF               
	RETURN

INITIAL_LCD:	CALL	FUNCTION_INIT		; Initialize by instruction 
	CALL	FUNCTION_SET		; True initial start here
	CALL	ENTRY_MODE_SET
	CALL	DISPLAY_CONTROL
	RETURN

FUNCTION_INIT:	MOVLW	0x33			; First & second write
	CALL	WRITE_COMMAND2
	MOVLW	0x32			; Third write and true initial
	GOTO	WRITE_COMMAND

FUNCTION_SET:	MOVLW	FUNCTION_VALUE
	GOTO	WRITE_COMMAND2

ENTRY_MODE_SET:	MOVLW	ENTRY_VALUE
	GOTO	WRITE_COMMAND

DISPLAY_CONTROL:	MOVLW	DISPLAY_VALUE
	GOTO	WRITE_COMMAND

WRITE_DATA:	HIGH_RS
	GOTO	WRITE

WRITE_COMMAND:	LOW_RS

WRITE:	MOVWF	DATA_BUS
	MOVWF	TEMP_DATA
	LOW_E		
	CALL	DELAY40U
	HIGH_E
	SWAPF	TEMP_DATA,0
	MOVWF	DATA_BUS
	NOP
	LOW_E
	CALL	DELAY40U

DISABLE:	HIGH_E
	RETURN

WRITE_COMMAND2:	LOW_RS
	MOVWF	DATA_BUS
	MOVWF	TEMP_DATA
	LOW_E		
	CALL	DELAY5M
	HIGH_E
	SWAPF	TEMP_DATA,0
	MOVWF	DATA_BUS
	NOP
	LOW_E
	CALL	DELAY5M
	HIGH_E
	RETURN

CLEAR_SCREEN:	MOVLW	B'00000001'	
	GOTO	WRITE_COMMAND2

RETURN_HOME:	MOVLW	B'00000010' 		; RETURN HOME
	GOTO	WRITE_COMMAND2

DELAY5M:	MOVLW	.7			; Delay ~ 5.4 ms
	GOTO	DELAY	
DELAY15M:	MOVLW	.20			; Delay ~ 15 ms for internal reset
DELAY:	MOVWF	X	
	CLRF	Y
	DECFSZ	Y,1
	GOTO	$-1
	DECFSZ	X,1
	GOTO	$-3
	RETURN
DELAY40U:	MOVLW	.12			; Delay ~ 40 us
	MOVWF	X
	DECFSZ	X,1
	GOTO	$-1
	RETURN
DIV2416:	MOVLW	0x0F			; Unsign divide 24/16
	MOVWF	AARGB0
	MOVLW	0x42
	MOVWF	AARGB1
	MOVLW	0x40
	MOVWF	AARGB2
	CLRF	REMB0
	CLRF	REMB1
	CLRF	TEMP
	RLF	AARGB0,0 
	RLF	REMB1,1 
	MOVF	BARGB1,0 
	SUBWF	REMB1,1
	MOVF	BARGB0,0
	SKPC	
	INCFSZ	BARGB0,0	
	SUBWF	REMB0,1	
	CLRW
	SKPC		
	MOVLW	0x01
	SUBWF	TEMP,1	
	RLF	AARGB0,1	
	MOVLW	0x07
	MOVWF	LOOPCOUNT	
LOOPU2416A:	RLF	AARGB0,0	
	RLF	REMB1,1	
	RLF	REMB0,1	
	RLF	TEMP,1	
	MOVF	BARGB1,0	
	BTFSS	AARGB0,0	
	GOTO	UADD46LA	
	SUBWF	REMB1,1	
	MOVF	BARGB0,0	
	SKPC		
	INCFSZ	BARGB0,0	
	SUBWF	REMB0,1	
	CLRW	
	SKPC		
	MOVLW	0x01
	SUBWF	TEMP,1	
	GOTO	UOK46LA	
UADD46LA	ADDWF	REMB1,1	
	MOVF	BARGB0,0	
	SKPNC		
	INCFSZ	BARGB0,0	
	ADDWF	REMB0,1	
	CLRW
	SKPNC		
	MOVLW	0x01
	ADDWF	TEMP,1	
UOK46LA	RLF	AARGB0,1	
	DECFSZ	LOOPCOUNT,1	
	GOTO	LOOPU2416A	
	RLF	AARGB1,0	
	RLF	REMB1,1	
	RLF	REMB0,1	
	RLF	TEMP,1	
	MOVF	BARGB1,0	
	BTFSS	AARGB0,0	
	GOTO	UADD46L8	
	SUBWF	REMB1,1	
	MOVF	BARGB0,0	
	SKPC		
	INCFSZ	BARGB0,0	
	SUBWF	REMB0,1	
	CLRW
	SKPC		
	MOVLW	0x01
	SUBWF	TEMP,1	
	GOTO	UOK46L8	
UADD46L8	ADDWF	REMB1,1	
	MOVF	BARGB0,0	
	SKPNC		
	INCFSZ	BARGB0,0	
	ADDWF	REMB0,1	
	CLRW
	SKPNC		
	MOVLW	0x01
	ADDWF	TEMP,1	
UOK46L8	RLF	AARGB1,1	
	MOVLW	0x07
	MOVWF	LOOPCOUNT	
LOOPU2416B	RLF	AARGB1,0	
	RLF	REMB1,1	
	RLF	REMB0,1	
	RLF	TEMP,1	
	MOVF	BARGB1,0	
	BTFSS	AARGB1,0	
	GOTO	UADD46LB	
	SUBWF	REMB1,1	
	MOVF	BARGB0,0	
	SKPC		
	INCFSZ	BARGB0,0	
	SUBWF	REMB0,1	
	CLRW	
	SKPC		
	MOVLW	0x01
	SUBWF	TEMP,1	
	GOTO	UOK46LB	
UADD46LB	ADDWF	REMB1,1	
	MOVF	BARGB0,0	
	SKPNC		
	INCFSZ	BARGB0,0	
	ADDWF	REMB0,1	
	CLRW
	SKPNC		
	MOVLW	0x01
	ADDWF	TEMP,1		
UOK46LB	RLF	AARGB1,1	
	DECFSZ	LOOPCOUNT,1	
	GOTO	LOOPU2416B	
	RLF	AARGB2,0	
	RLF	REMB1,1	
	RLF	REMB0,1	
	RLF	TEMP,1	
	MOVF	BARGB1,0	
	BTFSS	AARGB1,0	
	GOTO	UADD46L16	
	SUBWF	REMB1,1	
	MOVF	BARGB0,0	
	SKPC		
	INCFSZ	BARGB0,0	
	SUBWF	REMB0,1	
	CLRW
	SKPC		
	MOVLW	0x01
	SUBWF	TEMP,1	
	GOTO	UOK46L16	
UADD46L16	ADDWF	REMB1,1	
	MOVF	BARGB0,0	
	SKPNC		
	INCFSZ	BARGB0,0	
	ADDWF	REMB0,1	
	CLRW
	SKPNC		
	MOVLW	0x01
	ADDWF	TEMP,1		
UOK46L16	RLF	AARGB2,1	
	MOVLW	0x07
	MOVWF	LOOPCOUNT	
LOOPU2416C	RLF	AARGB2,0	
	RLF	REMB1,1	
	RLF	REMB0,1	
	RLF	TEMP,1	
	MOVF	BARGB1,0	
	BTFSS	AARGB2,0	
	GOTO	UADD46LC	
	SUBWF	REMB1,1	
	MOVF	BARGB0,0	
	SKPC		
	INCFSZ	BARGB0,0	
	SUBWF	REMB0,1	
	CLRW	
	SKPC		
	MOVLW	0x01
	SUBWF	TEMP,1	
	GOTO	UOK46LC	
UADD46LC	ADDWF	REMB1,1	
	MOVF	BARGB0,0	
	SKPNC		
	INCFSZ	BARGB0,0	
	ADDWF	REMB0,1	
	CLRW
	SKPNC		
	MOVLW	0x01
	ADDWF	TEMP,1	
UOK46LC	RLF	AARGB2,1	
	DECFSZ	LOOPCOUNT,1	
	GOTO	LOOPU2416C	
	BTFSC	AARGB2,0	
	GOTO	UOK46L
	MOVF	BARGB1,0	
	ADDWF	REMB1,1	
	MOVF	BARGB0,0	
	SKPNC		
	INCFSZ	BARGB0,0	
	ADDWF	REMB0,1	
UOK46L	RETURN	
	
	END
