;***************************************************************************
;Program :	1kHz pulse generator
;Description :	Using interrupt in compare module for generating 1kHz pulse 
;Processor :	PIC16F628-04 MHz 
;filename :	hlab2603.asm
;***************************************************************************
	LIST	P=16F628,W=-302
PORTB	EQU	0x06			; Bank 0
INTCON	EQU	0x0B			; Every bank
PIR1	EQU	0x0C
TMR1L	EQU	0x0E
TMR1H	EQU	0x0F
T1CON	EQU	0x10
CCPR1L	EQU	0x15
CCPR1H	EQU	0x16
CCP1CON	EQU	0x17
TEMP	EQU	0x70			; Common RAM
TRISB	EQU	0x86			; Bank 1
PIE1	EQU	0x8C
TMR1ON	EQU	0
CCP1IE	EQU	2
CCP1IF	EQU	2
GIE	EQU	7
PEIE	EQU	6

	__CONFIG	0x3F61

	ORG	0x0000
	GOTO	MAIN
	
	ORG	0x0004
ISR	BCF	T1CON,TMR1ON		; Turn off Timer1
	BCF	PIR1,CCP1IF 		; Clear interrupt flag
	CLRF	TMR1H
	CLRF	TMR1L
	BTFSS	CCP1CON,0
	GOTO	TOGGLE
	BCF	CCP1CON,0
	GOTO	SKIP
TOGGLE	BSF	CCP1CON,0
SKIP	BSF	T1CON,TMR1ON
	RETFIE					; GIE = 1 after return
MAIN
	BANKSEL	TRISB				; Initial port
	BCF	TRISB,3			; RB3 as output
	BANKSEL	CCPR1H			; Bank select	
	MOVLW	0x01				; T = 0.5 ms for 1 kHz.
	MOVWF	CCPR1H   
	MOVLW	0xF3
	MOVWF	CCPR1L
	CLRF	T1CON				; Prescaler 1:1, internal clock
	BANKSEL	PIE1	   			; Bank select		
	BSF	PIE1,CCP1IE 		; Interrupt from compare	
	BSF	INTCON,PEIE 		; Peripheral enable
	BANKSEL	CCP1CON
	MOVLW	B'00001001' 		; Clear on match mode
	MOVWF	CCP1CON
	CALL	ISR
WAIT	GOTO	WAIT

	END

