;*************************************************************************
;Program :	PWM genrator with duty cycle adjustable
;Description :	Using CCP1 module for generating 10kHz PWM signal 
;		which duty cycle adjustable
;filename :	hlab2702.asm
;*************************************************************************
	LIST 	P=16F628,W=-302
;** fPWM = 10kHz.
;** DUTY = 1-99%

PCL	EQU	0x02
PORTA	EQU	0x05
PORTB	EQU	0x06	
TMR2	EQU	0x11
T2CON	EQU	0x12
CCPR1L	EQU	0x15
CCP1CON	EQU	0x17
CMCON	EQU	0x1F

TRISA	EQU	0x85
TRISB	EQU	0x86	
PR2	EQU	0x92

CCP1X	EQU	5
CCP1Y	EQU	4

      	CBLOCK	0x70
       	BUFFER1,BUFFER2,BUFFER_DEC,BUFFER_HEX
        	ENDC

#DEFINE	SW_DEC	PORTA,0		
#DEFINE	SW_INC	PORTA,1

	ORG	0x000
	__CONFIG	0x3F61
INIT_PORT
	MOVLW	.50	
	MOVWF	BUFFER_DEC			; For displaying 50% duty cycle
MOVLW	0x50		
	MOVWF	BUFFER_HEX

	MOVLW	.7
	MOVWF	CMCON			; RA3-RA0 as digital I/O	
	CLRF	PORTB			; Clear display after reset
	BANKSEL	TRISA			; Bank select
	MOVLW	B'11110011'
	MOVWF	TRISA
	CLRF	TRISB			; PORTB as output
		
	MOVLW	.24				; Period      
	MOVWF	PR2

	BANKSEL	PORTA	
	MOVLW	B'11110111' 			; For scaning display
	MOVWF	PORTA
	
	MOVLW	B'00001111'			; Select PWM mode
	MOVWF	CCP1CON	
	
	CALL	CONTROL_DUTY  

	CLRF	TMR2				; Clear Timer2
	MOVLW	B'00000001'			; Prescaler = 1:4  
	MOVWF	T2CON			; Timer2 off
	BSF	T2CON,2			; Start Timer2
READ_SW_DEC	CALL	DISPLAY
	BTFSC	SW_DEC	
	GOTO	READ_SW_INC
	CALL	DECREASE	
	CALL	DISPLAY
	BTFSS	SW_DEC			; Wait for DEC switch released
	GOTO	$-2
READ_SW_INC	CALL	DISPLAY
	BTFSC	SW_INC
	GOTO	READ_SW_DEC
	CALL	INCREASE
	CALL	DISPLAY
	BTFSS	SW_INC			; Wait for INC switch released
	GOTO	$-2 
	GOTO	READ_SW_DEC
DECREASE	DECF	BUFFER_DEC,1
	SKPZ
	GOTO	LOOP1
	BSF	BUFFER_DEC,0 			; Buffer = .1
LOOP1       	DECF	BUFFER_HEX,1
	BZ	LOOP2
	CALL	CONTROL_DUTY
	CALL	DEC_ADJUST
	RETURN

LOOP2	BSF	BUFFER_HEX,0 			; BUFFER_HEX = 0x01
	GOTO	$-4
	RETURN

INCREASE	INCF	BUFFER_DEC,1     
	MOVLW	.100
	SUBWF	BUFFER_DEC,0
	SKPZ
	GOTO	LOOP3	
	MOVLW	.99
	MOVWF	BUFFER_DEC	     		; Buffer = .99
LOOP3	INCF	BUFFER_HEX,1
	MOVLW	0x9A
	SUBWF	BUFFER_HEX,0
	BZ	LOOP4
	CALL	CONTROL_DUTY
	CALL	INC_ADJUST
	RETURN

LOOP4	MOVLW	0x99	    			; BUFFER_HEX = 0x99
	MOVWF	BUFFER_HEX
	GOTO	$-5
	RETURN

DISPLAY	MOVFW	BUFFER_HEX
	CALL	TABLE	
	SWAPF	BUFFER_HEX,0
	CALL	TABLE
	RETURN

TABLE	ANDLW	B'00001111'
	CALL	CHAR
	MOVWF	PORTB
	COMF	PORTA,1
DELAY	CLRF	BUFFER1
	DECFSZ	BUFFER1,1
	GOTO	$-1
	RETURN
CHAR	ADDWF	PCL,1
	DT	0F6,060,0D5,0F1,063,0B3,0B7,0E0,0F7,0F3
		;CHARACTER=> 0 , 1 , 2 , 3 , 4 , 5 , 6 , 7 , 8 , 9	
INC_ADJUST	MOVFW	BUFFER_HEX
	ANDLW	B'00001111'			; Mask bit
	SUBLW	0x09	 
	SKPNC						; Skip if 9 < W
	RETURN
	MOVLW	.6
	ADDWF	BUFFER_HEX,1
	RETURN	
	
DEC_ADJUST	MOVFW	BUFFER_HEX
	ANDLW	B'00001111'			; Mask bit
	SUBLW	0x09	
	SKPNC						; Skip if 9 < W
	RETURN
	MOVLW	.6
	SUBWF	BUFFER_HEX,1
	RETURN
		
CONTROL_DUTY	MOVFW	BUFFER_DEC
	MOVWF	BUFFER2
	CLRC		    				; Clear Carry flag
	RRF	BUFFER2,1
	BC	SET_CCP1Y
	BCF	CCP1CON,CCP1Y   		; Bit CCP1Y = 0		
	GOTO	NEXT1
SET_CCP1Y	BSF	CCP1CON,CCP1Y   		; Bit CCP1Y = 1
NEXT1	CLRC		    	
	RRF	BUFFER2,1
	BC	SET_CCP1X
	BCF	CCP1CON,CCP1X   		; Bit CCP1X = 0
	GOTO	NEXT2
SET_CCP1X	BSF	CCP1CON,CCP1X   		; Bit CCP1X = 1
NEXT2	MOVFW	BUFFER2
	MOVWF	CCPR1L	   		; HIGH 8 BIT	
	RETURN		

	END
