;*************************************************************
; Program : 	Serial data receiving
; Description :	Using USART module for RS-232 receiving data 
; Filename : 	hlab2802.asm
;*************************************************************
	LIST	P=16F628,W=-302
PORTA	EQU	0x05				; Bank 0
PORTB	EQU	0x06
PIR1	EQU	0x0C	
RCSTA	EQU	0x18
RCREG	EQU	0x1A
CMCON	EQU	0x1F
TRISA	EQU	0x85				; Bank 1
TRISB	EQU	0x86
TXSTA	EQU	0x98
SPBRG	EQU	0x99
TXIF	EQU	4				; Bit
TXEN	EQU	5
SYNC	EQU	4
BRGH	EQU	2
SPEN	EQU	7
CREN	EQU	4
FERR	EQU	2
OERR	EQU	1
RCIF	EQU	5
DATA1	EQU	0x70				; GP RAM 
VAR	EQU	0x71

	__CONFIG	0x3F61			; XT-4 MHz, WDT off
	ORG	0x0000
	MOVLW	.7
	MOVWF	CMCON				; RA3-RA0 as digial I/O
	BANKSEL	TRISB				; Bank 1
	MOVLW	B'00001111'   		; RB7-RB4 as output
	MOVWF	TRISB
	MOVLW	B'11110000'		; RA3-RA0 as output
	MOVWF	TRISA
	
	MOVLW	.25
	MOVWF	SPBRG				; baudrate = 9600 bps
	BSF	TXSTA,BRGH		; HI SPEED baudrate
	BANKSEL	RCSTA				; Bank 0
	CLRF	PORTA				; Turn-off LED
	CLRF	PORTB				; Turn-off LED
	BSF	RCSTA,SPEN     		; Serial port enable
	BSF	RCSTA,CREN		; Begin receive
WAIT	BTFSC	RCSTA,FERR		; Check frame error after received,
	GOTO	FRAME_ERROR		; If no then skip 
	BTFSS	PIR1,RCIF      		; If have data then skip
	GOTO	WAIT				; No data then waiting
	MOVFW	RCREG				; Move data to w 
	MOVWF	PORTB				; Show data on LED
	MOVWF	PORTA

;	CALL	DELAY				; Delay for overrun testing

	BTFSS	RCSTA,OERR  
	GOTO	WAIT				; No overrun then come back
	GOTO	OVERRUN_ERROR
FRAME_ERROR    
	BANKSEL	TRISB				; Bank 1
	BCF	TRISB,3			; RB3 as output
	BANKSEL	PORTB				; Bank 0
	BSF	PORTB,3			; Frame error occured	
	GOTO	$

OVERRUN_ERROR	
	BANKSEL	TRISB				; Bank 1
	BCF	TRISB,0	
	BANKSEL	PORTB	; Bank 0
	BSF	PORTB,0			; Overrun error occured
	GOTO	$

DELAY	CLRF	VAR				; Delay ~ 94.72 ms
	MOVLW	.123
	MOVWF	VAR+1
	DECFSZ	VAR,F
	GOTO	$-1
	DECFSZ	VAR+1,F
	GOTO	$-3
	RETURN
	END

