;*************************************************************
; Program : 	Serial data transceiving
; Description :	Using USART module for RS-232 receiving and 
;		transmitting data from PCs serial port 
; Filename : 	hlab2803.asm
;*************************************************************
	LIST	P=16F628,W=-302

PCL	EQU	0x02	; Bank 0
PORTB	EQU	0x06	
INTCON	EQU	0x0B
PIR1	EQU	0x0C	
RCSTA	EQU	0x18
TXREG	EQU	0x19
RCREG	EQU	0x1A

TRISB	EQU	0x86	; Bank 1
PIE1	EQU	0x8C
TXSTA	EQU	0x98
SPBRG	EQU	0x99

CHAR	EQU	0x70	; RAM
COUNTER	EQU	0x71

BRGH	EQU	2	; Bit
TXIF	EQU	4
CREN	EQU	4
RCIE	EQU	5
RCIF	EQU	5
TXEN	EQU	5
PEIE	EQU	6
SPEN	EQU	7
GIE	EQU	7
;================= CONTROL CHARACTER =================================
LF	SET	0x0A    		; Line feed => Next line
FF	SET	0x0C    		; Form feed => New page
CR	SET	0x0D    		; Carriage return => Begining of the same line
;=====================================================================
	__CONFIG	0x3F61			; XT-4 MHz, WDT off

	ORG	0x0000			; Reset vector
	GOTO	MAIN

	ORG	0x0004			; Interrupt vector
	GOTO	ISR

MAIN	
	BANKSEL	TRISB				; Bank 1
	BSF	TRISB,2			; RB2 as input (TX pin)
	BSF	TRISB,1			; RB1 as input (RX pin)	
	
	BSF	PIE1,RCIE			; RX interrupt enable
	BSF	INTCON,PEIE 			; Peripheral interrupt enable				
	MOVLW	.12
	MOVWF	SPBRG				; Baudrate = 19200 bps
	BSF 	TXSTA,BRGH			; Hi-speed baudrate
	BSF	TXSTA,TXEN			; TX enable
	
	BANKSEL	RCSTA				; Bank 0
	BSF	RCSTA,CREN			; RX enable
	BSF	RCSTA,SPEN  			; Serial port enable	
	BSF	INTCON,GIE			; Interrupt enable

	CALL	SEND_DATA
	GOTO	$					; Assume this is any job 

SEND_DATA	MOVLW	.104
	MOVWF	COUNTER
	CLRF	CHAR
LOOP1	MOVFW	CHAR
	CALL	TABLE	
	CALL	TX
	INCF	CHAR,1
	DECFSZ	COUNTER,1	
	GOTO	LOOP1
	RETURN

TABLE	ADDWF	PCL,1
	DT	FF,LF,"NX-84 V2.0",LF,CR
	DT	"THE PICmicro TRAINING BOARD",LF,CR
	DT	"(C) INNOVATIVE EXPERIMENT,BANGKOK THAILAND",LF,LF,CR
	DT	"PRESS ANY KEY..."

TX	BTFSS	PIR1,TXIF			; Skip if TXREG empty
	GOTO	$-1				; Not empty then wait
	MOVWF	TXREG				; Send data to RS-232	
	RETURN
ISR	MOVFW	RCREG				; Read buffer1
	CALL	TX
READ_DATA	BTFSS	PIR1,RCIF			; If not empty then read buffer2
	RETFIE
	MOVFW	RCREG				; Read buffer2
	CALL	TX
	GOTO	READ_DATA			; If not empty then read again


	END
