;********************************************************************
; Program : 24C16 reading data -1
; Description : Demonstrate reading data from 24C16 by random read operation
; Filename : hlab3302.asm
;********************************************************************
	LIST	P=16F628,W=-302

PORTA	EQU	0x05				
PORTB  	EQU	0x06
TRISA	EQU	0x85
TRISB	EQU	0x86
CMCON	EQU	0x1F

WRITE_BUFFER  EQU	0x70		; COMMON RAM 
READ_BUFFER	EQU	0x71
COUNT8	EQU	0x72

; RA1 = SCL line
; RA0 = SDA line

	ORG	0x0000
  	__CONFIG	0x3F61 		; XT-OSC 4 MHz., WDT-off
;**********************************************************************
	MOVLW	.7
	MOVWF	CMCON		; RA1-RA0 = digital I/O
	BANKSEL	TRISB		; Access TRISB
	CLRF	PORTB		; PORTB as output
	BANKSEL	PORTB		; Access PORTB	
	CLRF	PORTB		; Clear  PORTB

FIRST_INIT	CALL	START		; Start communication
	MOVLW	B'10100000' 	; Send slave addresses,block0,R/W = 0
	CALL	CONTROL_BYTE
	MOVLW	0x00		; Read @ADDR = 0x00, block 0 
	CALL	WORD_ADDRESS

SECOND_INIT	CALL	START		; Start again
	MOVLW	B'10100001' 	; Send slave address, block 0, R/W = 1
	CALL	CONTROL_BYTE
	CALL	READ_DATA   		; Read data
	CALL	SEND_NACK		; Send NACK
	CALL	STOP		; stop I2C bus 

	MOVFW	READ_BUFFER
	MOVWF	PORTB
	
	GOTO	$		; Endless loop, end of job
	
;************************  READ SUBROUTINE  ***************************
START	CALL	SDA_HIGH		; Release  SDA line
	CALL	SCL_HIGH	
	BTFSS	PORTA,0		; SCL line free ?
	GOTO	$-1		; No, it busy
	BTFSS	PORTA,1		; SDA line free ?
	GOTO	$-1		; No, it busy
	CALL	SDA_LOW		; Start I2C bus	
	CALL	SCL_LOW
	RETURN		
					
CONTROL_BYTE
WORD_ADDRESS
WRITE_DATA
	CALL	WRITE		; Write data or address
	CALL	READ_ACK		; Check ACK.
	RETURN

WRITE	MOVWF	WRITE_BUFFER	; Put message in buffer
	MOVLW	.8
	MOVWF	COUNT8		; 8 loops
	CALL	SCL_LOW
WRITE_LOOP	RLF	WRITE_BUFFER,1 ; Send MSB first
	BC	SEND_1	
SEND_0	CALL	SDA_LOW		; Send logic low on SDA
	CALL	SCL_PULSE		; Send clock on SCL
	GOTO	CHECK		
SEND_1	CALL	SDA_HIGH		; Send logic high on SDA
	CALL	SCL_PULSE		; Send clock on SCL
CHECK	DECFSZ	COUNT8,1		; write complete ?
	GOTO	WRITE_LOOP		; No, write next bit 
	CALL	SDA_HIGH		; Release  SDA line
	RETURN			; Yes, return

READ_ACK	CALL	SDA_HIGH		; Release  SDA line
	CALL	SCL_LOW	
	CALL	SCL_HIGH	
	BTFSC	PORTA,0		; ACK. ?
	GOTO	STOP		; NACK, then stop 
	BTFSS	PORTA,1		; SCL line busy ?
	GOTO	$-1		; Yes
	CALL	SCL_LOW
	RETURN
SEND_NACK	CALL	SCL_LOW
	CALL	SDA_HIGH 		; Send NACK	
	CALL	SCL_HIGH
	CALL	SCL_LOW
	RETURN
READ_DATA	MOVLW	.8
	MOVWF	COUNT8
	CALL	SCL_LOW
	CALL	SDA_HIGH		; Release  SDA line
READ_LOOP	CALL	SCL_HIGH		; Read data here
	BTFSC	PORTA,0		; Read data on SDA line
	GOTO	RECEIVE_1		; Data = 1
RECEIVE_0	CLRC			; Data = 0
	GOTO	ROTATE_DATA
RECEIVE_1	SETC
ROTATE_DATA	RLF	READ_BUFFER,1
	CALL	SCL_LOW
	DECFSZ	COUNT8,1	
	GOTO	READ_LOOP
	RETURN	

STOP	CALL	SCL_LOW	
	CALL	SDA_LOW
	CALL	SCL_HIGH
	CALL	SDA_HIGH		; Release  SDA line 
	CALL	DELAY5	
	RETURN
	
SCL_HIGH		
	BANKSEL	TRISA		; Bank 1
	BSF	TRISA,1		; SCL = high
	CALL	DELAY5		; Call = 2 us
	BANKSEL	PORTA		; Bank 0
	RETURN		
SCL_LOW		
	BANKSEL	TRISA		; Bank 1 
	BCF	TRISA,1		; RA1 as output
	BANKSEL	PORTA		; Bank 0
	BCF	PORTA,1		; SCL = low
	CALL	DELAY5	
	RETURN		
SDA_HIGH	
	BANKSEL	TRISA		; Bank 1
	BSF	TRISA,0		; SDA = high
	BANKSEL	PORTA		; Bank 0
	RETURN
SDA_LOW		
	BANKSEL	TRISA		; Bank 1
	BCF	TRISA,0		; RA0 as output
	BANKSEL	PORTA		; Bank 0
	BCF	PORTA,0		; SDA = low
	RETURN

DELAY5	NOP			; 1 us
	GOTO	$+1		; 2 us	
	RETURN			; 2 us

SCL_PULSE	CALL	SCL_LOW
	CALL	SCL_HIGH
	CALL	SCL_LOW
	RETURN

	END
