;********************************************************************
; Program : 24C16 writing data -2
; Description : Demonstrate writing data to 24C16 by page write operation
; Filename : hlab3303.asm
;********************************************************************
	LIST	P=16F628,W=-302
PCL	EQU	0x02
PORTA	EQU	0x05
TRISA	EQU	0x85
CMCON	EQU	0x1F
WRITE_BUFFER  	EQU	0x70
READ_BUFFER	EQU	0x71
COUNT8	EQU	0x72
COUNT16	EQU	0x73
OFFSET	EQU	0x74

;RA1 = SCL line
;RA0 = SDA line
   	ORG	0x0000
 	__CONFIG	0x3F61	 		; XT-OSC 4 MHz., WDT-off
	MOVLW	.7
	MOVWF	CMCON			; RA1-RA0 = digital I/O
	BANKSEL	TRISA			; Access TRISA
	BCF	PORTA,2		; RA2 = write status bit
	BANKSEL	PORTA
	BCF	PORTA,2		; Clear  RA2
INITIAL1
	CALL	START			; Start communication
	MOVLW	B'10100000' 	; Send slave addresses,block0,R/W = 0
	CALL	CONTROL_BYTE
	MOVLW	0x00			; Address = 0x00, block 0 
	CALL	WORD_ADDRESS

	CLRF	OFFSET
	MOVLW	.16
	MOVWF	COUNT16		; Counter16
WRITE1	MOVFW	OFFSET
	CALL	DATA1
	CALL	WRITE_DATA	; Write TO EEPROM
	INCF	OFFSET,1
	DECFSZ	COUNT16,1
	GOTO	WRITE1
	CALL	STOP			; Send completed, start write here
	CALL	ACK_POLLING	; Wait for EEPROM writing
	NOP					; Write1 completed
INITIAL2	CALL	START			; Start communication
	MOVLW	B'10100010' 	; Send slave address, Block 1, R/W = 0
	CALL	CONTROL_BYTE
	MOVLW	0x00			; Address = 0x00, Block 1
	CALL	WORD_ADDRESS
	CLRF	OFFSET
	MOVLW	.16
	MOVWF	COUNT16		; Counter16
WRITE2	MOVFW	OFFSET
	CALL	DATA2
	CALL	WRITE_DATA	; Write TO EEPROM
	INCF	OFFSET,1
	DECFSZ	COUNT16,1
	GOTO	WRITE2
	CALL	STOP			; Send completed, start write here
	CALL	ACK_POLLING 	; Wait for EEPROM writing
	BSF	PORTA,2		; Write2 completed
	GOTO	$				; Endless loop
	
DATA1	ADDWF	PCL,1
	DT	0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07
	DT	0x08,0x09,0x0A,0x0B,0x0C,0x0D,0x0E,0x0F

DATA2	ADDWF	PCL,1
	DT	0x10,0x11,0x12,0x13,0x14,0x15,0x16,0x17
	DT	0x18,0x19,0x1A,0x1B,0x1C,0x1D,0x1E,0x1F		
	
;************************  WRITE SUBROUTINE  **************************
START	CALL	SDA_HIGH		; Release  SDA line
	CALL	SCL_HIGH	
	BTFSS	PORTA,0		; SCL line free ?
	GOTO	$-1			; No, it busy
	BTFSS	PORTA,1		; SDA line free ?
	GOTO	$-1			; No, it busy
	CALL	SDA_LOW		; Start I2C bus	
	CALL	SCL_LOW
	RETURN		
CONTROL_BYTE
WORD_ADDRESS
WRITE_DATA
	CALL	WRITE			; Write data or address
	CALL	READ_ACK		; Check ACK.
	RETURN

WRITE	MOVWF	WRITE_BUFFER	; Put message in buffer
	MOVLW	.8
	MOVWF	COUNT8	; 8 loops
	CALL	SCL_LOW
WRITE_LOOP	RLF	WRITE_BUFFER,1 ; Send MSB first
	BC	SEND_1	
SEND_0	CALL	SDA_LOW		; Send logic low on SDA
	CALL	SCL_PULSE		; Send clock on SCL
	GOTO	CHECK		
SEND_1	CALL	SDA_HIGH		; Send logic high on SDA
	CALL	SCL_PULSE		; Send clock on SCL
CHECK	DECFSZ	COUNT8,1		; Write complete ?
	GOTO	WRITE_LOOP	; No, write next bit 
	CALL	SDA_HIGH		; Release  SDA line
	RETURN					; Yes, return
	
READ_ACK	CALL	SDA_HIGH		; Release  SDA line
	CALL	SCL_LOW	
	CALL	SCL_HIGH	
	BTFSC	PORTA,0		; ACK. ?
	GOTO	STOP			; NACK, then stop 
	BTFSS	PORTA,1		; SCL line busy ?
	GOTO	$-1			; Yes
	CALL	SCL_LOW
	RETURN
ACK_POLLING	CALL	SCL_LOW
	CALL	START
	MOVLW	B'10100000'  
	CALL	WRITE			; Send slave address
WRITE_WAIT	CALL	SDA_HIGH		; Release  SDA line
	CALL	SCL_LOW
	CALL	SCL_HIGH	
	BTFSC	PORTA,0			; ACK. ?
	GOTO	ACK_POLLING		; NACK
	CALL	SCL_LOW			; ACK. OK, DO next operation
	RETURN

STOP	CALL	SCL_LOW	
	CALL	SDA_LOW
	CALL	SCL_HIGH
	CALL	SDA_HIGH			; Release  SDA line 
	CALL	DELAY5	
	RETURN
	
SCL_HIGH		
	BANKSEL	TRISA				; Bank 1
	BSF	TRISA,1			; SCL = high
	CALL	DELAY5				; Call = 2 us
	BANKSEL	PORTA				; Bank 0
	RETURN		
SCL_LOW		
	BANKSEL	TRISA				; Bank 1 
	BCF	TRISA,1			; RA1 as output
	BANKSEL	PORTA				; Bank 0
	BCF	PORTA,1			; SCL = low
	CALL	DELAY5	
	RETURN		
SDA_HIGH	
	BANKSEL	TRISA				; Bank 1
	BSF	TRISA,0			; SDA = high
	BANKSEL	PORTA				; Bank 0
	RETURN
SDA_LOW		
	BANKSEL	TRISA				; Bank 1
	BCF	TRISA,0			; RA0 as output
	BANKSEL	PORTA				; Bank 0
	BCF	PORTA,0			; SDA = low
	RETURN

DELAY5	NOP						; 1 us
	GOTO	$+1				; 2 us	
	RETURN						; 2 us

SCL_PULSE	CALL	SCL_LOW
	CALL	SCL_HIGH
	CALL	SCL_LOW
	RETURN

	END
