;*************************************************************
; Program : ADC0832 experiment
; Description : Demonstrate interfacing ADC0832 with PIC16F628
; Filename : hlab3501.asm
;*************************************************************
  	LIST 	P=16F628,W=-302
	__CONFIG	0X3F61	

;-------------------- BYTE DEFINE -----------------------;

STATUS	EQU	0X03	; Bank 0
PORTA	EQU	0X05
PORTB	EQU	0X06
CMCON	EQU	0X1F

TRISA	EQU	0X85	; Bank 1
TRISB	EQU	0X86

SAVE	EQU	0X70	; RAM
COUNT	EQU	0X71
TEMP	EQU	0X72
MCOUNT	EQU	0X73
NCOUNT	EQU	0X74

;------------------ BIT DEFINE -----------------------;

C	EQU	0
Z	EQU	2
RP0	EQU	5
W	EQU	0
F	EQU	1
CS	EQU	2	; RA2
CLK	EQU	1	; RA1
DI	EQU	0	; RA0
	
	ORG	0X0000

;----------------------- INITIAL ---------------------;

START	MOVLW	.7
	MOVWF	CMCON	; RA3-RA0 as digital I/O

	BSF	STATUS,RP0	; Bank 1
	MOVLW	B'11111001'	; RA2-RA1 = OUTPUT
	MOVWF	TRISA
	CLRF	TRISB	; PORTB = OUTPUT

	BCF	STATUS,RP0	; Bank 0
	MOVLW	B'00001100'	; START
	MOVWF	PORTA
	NOP

;---------------------- SET COMMAND ------------------;

CONVERT	BSF	STATUS,RP0	; Bank 1
	BCF	TRISA,0	; RA0 = OUTPUT
	BCF	STATUS,RP0	; Bank 0
	
 	BCF	PORTA,CS	; CS LOW 
	NOP
	BSF     	PORTA,DI	; START BIT
	NOP
	BSF	PORTA,CLK 	; -----------
	NOP		; 1'ST CLOCK
	BCF	PORTA,CLK 	; -----------
	NOP

; CHANGE NEXT LINE FOR SELECT MODE

	BSF	PORTA,DI	; 0=DIFF, 1=SINGLE

	NOP
	BSF	PORTA,CLK 	; -----------
	NOP		; 2'ND CLOCK
	BCF	PORTA,CLK 	; -----------  
	NOP
	BCF	PORTA,DI	; SELECT CH0
	NOP
	BCF	PORTA,CLK 	; -----------
	NOP		; 3'RD CLOCK
	BCF	PORTA,CLK	; -----------
	NOP

;--------------------- START CONVERSION -------------------

	BSF	PORTA,CLK 	; RISING CLOCK : START CONVERSION
	CALL 	SERIN	; READ SERIAL ADC SUB
	BSF	PORTA,CLK      ; ------------
	NOP		; LAST CLOCK
	BCF	PORTA,CLK	; ------------
	NOP
	BSF	PORTA,CS	; CS HIGH

;--------------------- DISPLAY DATA -----------------------

	MOVF	SAVE,W	; READ VALUE
	MOVWF	PORTB	; DISPLAY ADC VALUE
	CALL	DELAY	; DELAY
	GOTO	CONVERT	; READ AGAIN

;---------------- READ SERIAL ADC SUBRUTINE ---------------

SERIN	BSF	STATUS,RP0	; Bank 1
	BSF	TRISA,0	; RA0 = INPUT
	BCF	STATUS,RP0     ; Bank 0
	CLRF	SAVE	; CLEAR OLD DATA
	MOVLW	0X09	; 8 BIT COUNTER
	MOVWF	COUNT

GETBIT	BCF	PORTA,CLK	; FALLING 4'TH CLOCK
	NOP		; END OF CONVERSION
	MOVF	PORTA,W	; READ MSB FISRT
	MOVWF	TEMP	
	RRF	TEMP,F	; TEMP => CARRY FLAG
	RLF	SAVE,F	; CARRY FLAG => SAVE
	DECFSZ	COUNT,F	; COUNTER = 8 ?
	GOTO	SHIFT	; NO READ AGAIN

;-------------- 11 CLOCK FOR LSB FIRST VALUE --------------
 
	MOVLW	0X0B	
	MOVWF	COUNT	; 11 CLOCK 
CLOCK	BSF	PORTA,CLK 	; ------------
	NOP		; CLOCK
	BCF	PORTA,CLK	; ------------
	DECFSZ  COUNT,F	; CLOCK = 11 ?
	GOTO	CLOCK
	RETURN

SHIFT	BSF	PORTA,CLK	; READ AGAIN
	GOTO	GETBIT

;------------------- DELAY SUBRUTINE ----------------------

DELAY	MOVLW	0XFF
	MOVWF	MCOUNT
DECM	MOVLW	0XFF
	MOVWF	NCOUNT
DECN	DECFSZ	NCOUNT,F
	GOTO	DECN
	DECFSZ         MCOUNT,F
	GOTO	DECM
	RETURN

	END
